#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''
Python scraper for fetching Broadcast and Cable TV ratings from 
tvbythenumbers.zap2it.com

MIT License

Copyright (c) 2017 Sharidan Barboza

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
'''

import re

from .utils import *
from .constants import YESTERDAY, URL_FORMAT, BASE_URL, SEARCH_URL, PAGE_ERROR
from .search import SearchDaily
from .sorter import Sorter


class Entry(object):
    """A single row/entry in a cable or broadcast ratings chart."""

    def __init__(self, **kwargs):
        """Contains attributes for a row in a daily ratings chart.

        :param show: The name of the TV show.
        :param net: The name of the network.
        :param time: The time the show airs.
        :param rating: The percentage of all TV-households viewing show.
        :param viewers: The number of viewers in millions.
        :param share: For Broadcast only. The percentage of TV's currently on
                      that are viewing the show.
        """
        for key, value in kwargs.items():
            kwargs[key] = convert_float(safe_unicode(value))
        self.__dict__.update(kwargs)

    def __repr__(self):        
        """Format row for entry object in a ratings chart"""
        s = None

        try: 
            # Set width for network column (cable has longer width)
            width = 7 if hasattr(self, 'share') else 16

            s = '|{:<30.30s}|{:>10s}|'.format(self.show, self.time)
            s += '{0:>{1}.{2}}|'.format(self.net, width, width)
            s += '{:7.2f}|{:7.1f}|'.format(self.viewers, self.rating)

            # For broadcast only
            if hasattr(self, 'share'):
                s += '{:7.1f}|'.format(self.share)
        except TypeError:
            # Return None if entry cannot be represented
            s = None
        finally:
            # Return repr string
            return s

    def __getitem__(self, item):
        """Return entry object when accessed by index"""
        return self.__dict__[item]

    def get_json(self):
        """Represent entry object as a JSON string"""
        return to_json(self)


class Ratings(object):
    """Finds and parses charts from a ratings page."""

    def __init__(self, **kwargs):
        """Main parent class for fetching entries from a ratings chart.

        :param category: cable, final, or tv (non-final broadcast)
        :param show: Will only fetch data of a specific show.
        :param network: Will only fetch data of a specific network.
        :param limit: Will stop fetching data once the limit has been reached.
        :param date: Default - yesterday's date.
        """

        # Convert show and network attributes to lists
        for attr in ["show", "network"]:
            key = kwargs.get(attr)
            if key is not None and not isinstance(key, list):
                kwargs[attr] = [key]                            
        self.__dict__.update(kwargs)

        self.date_obj = convert_date(self.date)
        self.weekday = get_day(self.date_obj)
        self.soup = self._get_ratings_page()
        self.next_week = next_week(self.date_obj)
        self.last_week = last_week(self.date_obj)

        # After finding the page, grab the results
        if self._verify_page():
            self.entries = self.fetch_entries()
        else:
            raise PageNotFoundError(PAGE_ERROR)

    def sort(self, attr):
        """Sort the ratings based on an attribute"""
        self.entries = Sorter(self.entries, self.category, attr).sort_entries()
        return self

    def get_all(self, attr):
        """Returns a list of the requested attribute from all entries"""
        return [entry[attr] for entry in self.entries]

    def get_url(self):
        """Get the ratings page url"""
        return self.url

    def get_date(self):
        """Get the ratings page date"""
        return self.date

    def get_next_week(self):
        """Get the date object for next week"""
        return self.next_week

    def get_last_week(self):
        """Get the date object for last week"""
        return self.last_week

    def get_title(self):
        """Title is either the chart header for a cable ratings page or above
        the opening description for a broadcast ratings page.
        """
        if self.category == 'cable':
            strings = get_strings(self.soup, 'strong')
        else:
            strings = get_strings(self.soup, 'b')

        if len(strings) >= 1 and self.category == 'cable':
            return strings[0]
        elif len(strings) > 0 and 'Fast' in strings[-1]:
            return strings[0]

        return ''.join(strings)

    def get_json(self):
        """Serialize ratings object as JSON-formatted string"""
        ratings_dict = {
            'category': self.category,
            'date': self.date,
            'day': self.weekday,
            'next week': self.next_week,
            'last week': self.last_week,
            'entries': self.entries,
            'url': self.url
        }
        return to_json(ratings_dict)

    def get_rows(self):
        raise NotImplementedError('Must be overwritten by subclass.')

    def _build_url(self, shorten=True):
        raise NotImplementedError('Must be overwritten by subclass.')

    def fetch_entries(self):
        raise NotImplementedError('Must be overwritten in subclass.')

    def __iter__(self):
        for entry in self.entries:
            yield entry

    def __getitem__(self, item):
        return self.entries[item]

    def __len__(self):
        return len(self.entries)

    def _get_url_params(self, shorten=True):
        """Returns a list of each parameter to be used for the url format."""
        cable = True if self.category == 'cable' else False
        url_date = convert_month(self.date, shorten=shorten, cable=cable)

        return [
            BASE_URL,
            self.weekday.lower(),
            self.category + '-ratings',
            url_date.replace(' ', '-')
        ]

    def _match_query(self, show, net):
        return self._match_show(show) and self._match_net(net)

    def _match_show(self, show):
        """Match a query for a specific show/list of shows"""
        if self.show:
            return match_list(self.show, show)
        else:
            return True

    def _match_net(self, net):
        """Match a query for a specific network/list of networks"""
        if self.network:
            return match_list(self.network, net)
        else:
            return True

    def _verify_page(self):
        """Verify the ratings page matches the correct date"""
        title_date = self._get_date_in_title().lower()
        split_date = self.date.lower().split()
        split_date[0] = split_date[0][:3]
        return all(term in title_date for term in split_date)

    def _get_date_in_title(self):
        """Extract the date string from the title."""
        title = unescape_html(''.join(self.get_title()))

        # Extract string from header by getting last 3 words
        #date_string = ' '.join(self.get_title().split()[-3:])
        #return convert_string(date_string)
        return convert_string(title)

    def _get_ratings_page(self):
        """Do a limited search for the correct url."""
        # Use current posted date to build url
        self._build_url()
        soup = get_soup(self.url)
        if soup:
            return soup
        
        # Try building url again with unshortened month
        self._build_url(shorten=False)
        soup = get_soup(self.url)
        if soup:
            return soup

        # If not page is found, use search
        return SearchDaily(self.category, date=self.date).fetch_result()


class Cable(Ratings):
    """Ratings subclass that parses daily cable ratings charts."""

    def __init__(self, date=YESTERDAY, show=None, network=None, limit=None):
        """
        Cable shows are shows not belonging to a major broadcast network.
        By default, will output the top 100 cable shows for that day.
        """
        cable_dict = {
            'category': 'cable',
            'date': date,
            'show': show,
            'network': network,
            'limit': limit
        }

        try:
            Ratings.__init__(self, **cable_dict)
        except Exception:
            raise PageNotFoundError(PAGE_ERROR) from None

    def __repr__(self):
        s = 'Cable Ratings for {day}, {date}'.format(
            day=self.weekday, date=self.date.title())
        s += '\n|{:<30s}|{:<10s}|{:<16s}|{:<7s}|{:<7s}|'.format(
            'Show', 'Time', 'Network', 'Viewers', 'Rating')
        s += '\n+' + '-' * 74 + '+'

        for entry in self:
            try:
                s += '\n{0}'.format(entry)
            except TypeError:
                pass
        return s

    def _build_url(self, shorten=True):
        """Build the url for a cable ratings page"""
        self.url = URL_FORMAT.format(*self._get_url_params(shorten=shorten))

    def get_rows(self):
        """Get the rows from a cable ratings chart"""
        return self.soup.find_all('tr')[1:]

    def fetch_entries(self):
        """Fetch data and parse it to build a list of cable entries."""
        data = []
        for row in self.get_rows():
            # Stop fetching data if limit has been met
            if exceeded_limit(self.limit, len(data)):
                break

            entry = row.find_all('td') 
            entry_dict = {}

            show = entry[0].string
            net = entry[1].string
            if not self._match_query(show, net):
                continue

            entry_dict['show'] = show
            entry_dict['net'] = net
            entry_dict['time'] = entry[2].string

            if ',' in entry[3].string:
                entry_dict['viewers'] = entry[3].string.replace(',', '.')
            else:
                entry_dict['viewers'] = '0.' + entry[3].string
            entry_dict['rating'] = entry[4].string

            # Add data to create cable entry
            data.append(Entry(**entry_dict))

        return data


class Broadcast(Ratings):
    """Ratings subclass that parses daily broadcast ratings charts."""

    def __init__(self, date=YESTERDAY, show=None, network=None, limit=None, final=True):
        """
        Broadcast shows are shows belonging to the 5 major US broadcast
        networks: ABC, NBC, CBS, FOX, and the CW.

        Broadcast ratings can be split into two categories: final and
        non-final ("Fast-Affiliate"). Non-final ratings are posted earlier
        than final ratings.

        :param final: Fetch final or 'fast-affiliate' ratings.
        """
        broadcast_dict = {
            'category': 'final' if final else 'tv',
            'date': date,
            'show': show,
            'network': network,
            'limit': limit
        }

        try:
            Ratings.__init__(self, **broadcast_dict)
        except Exception:
            raise PageNotFoundError(PAGE_ERROR) from None

    def __repr__(self):
        if self.category == 'final':
            title_cat = 'Final Broadcast Ratings'
        else:
            title_cat = 'Fast Affiliate Broadcast Ratings'

        s = '{0} for {1}, {2}'.format(
            title_cat, self.weekday, self.date.title())
        s += '\n|{:<30s}|{:10s}|{:7s}|{:7s}|{:7s}|{:7s}|'.format(
            'Show', 'Time', 'Network', 'Viewers', 'Rating', 'Share')
        s += '\n+' + '-' * 73 + '+'

        for entry in self:
            try:
                s += '\n{0}'.format(entry)
            except TypeError:
                pass
        return s

    def _build_url(self, shorten=True):
        """Build the url for a broadcast ratings page"""
        url_order = self._get_url_params(shorten=shorten)

        # For fast ratings, switch weekday and category in url
        if self.category != 'final':
            url_order[1], url_order[2] = url_order[2], url_order[1]
        self.url = URL_FORMAT.format(*url_order)

    def get_rows(self):
        """Get the rows from a broadcast ratings chart"""
        table = self.soup.find_all('tr')[1:-3]
        return [row for row in table if row.contents[3].string]

    def fetch_entries(self):
        """Fetch data and parse it to build a list of broadcast entries."""
        current_time = ''

        data = []
        for row in self.get_rows():
            # Stop fetching data if limit has been met
            if exceeded_limit(self.limit, len(data)):
                break

            entry = row.find_all('td')
            entry_dict = {}

            show_time = entry[0].string
            if show_time and show_time != current_time:
                current_time = show_time
            if not show_time:
                show_time = current_time
            entry_dict['time'] = show_time

            show_string = entry[1].string.split('(')
            show = show_string[0][:-1]
            net = self._get_net(show_string)
            if not self._match_query(show, net):
                continue

            entry_dict['show'] = show
            entry_dict['net'] = net
            entry_dict['viewers'] = entry[3].string.strip('*')
            entry_dict['rating'], entry_dict['share'] = self._get_rating(entry)

            # Add data to initialize broadcast entry
            data.append(Entry(**entry_dict))

        return data

    def get_averages(self):
        """Get the broadcast network averages for that day.

        Returns a dictionary:
        key: network name
        value: sub-dictionary with 'viewers', 'rating', and 'share' as keys
        """
        networks = [unescape_html(n.string) for n in self.soup.find_all('td', width='77')]
        table = self.soup.find_all('td', style=re.compile('^font'))

        # Each element is a list split as [rating, share]
        rateshares = [r.string.split('/') for r in table[:5] if r.string]
        viewers = [v.string for v in table[5:] if v.string]
        averages = {}

        # Load the averages dict
        for index, network in enumerate(networks):
            viewer = convert_float(unescape_html(viewers[index]))
            rating = convert_float(unescape_html(rateshares[index][0]))
            share = convert_float(unescape_html(rateshares[index][1]))
            averages[network] = {'viewer': viewer, 'rating': rating, 'share': share}

        return averages

    def _get_net(self, entry):
        """Get the network for a specific row"""
        try:
            net = entry[1]
            return net[net.find('(')+1:net.find(')')]
        except IndexError:
            return None

    def _get_rating(self, entry):
        """Get the rating and share for a specific row"""
        r_info = ''
        for string in entry[2].strings:
            r_info += string
        rating, share = r_info.split('/')
        return (rating, share.strip('*'))





#!/usr/bin/env python\

from .constants import BASE_URL, SEARCH_URL, PAGE_ERROR
from .utils import PageNotFoundError, get_day, get_soup, convert_date, date_in_range

class SearchDaily(object):
    """Uses the search page to search for daily ratings pages based on
    specific category and date.
    """

    def __init__(self, category, date):
        """If requesting ratings within the last 2 days (especially on
        weekends) and page is not found, the ratings page has most likely
        not been posted yet.

        :param category: 'cable', broadcast', 'final' or 'tv' (non-final)
        :param date: Must be formatted as Month Date Year (e.g. January 6 2016)
        """
        self._assert_category(category)

        self.category = category
        self.date = date
        self.date_obj = convert_date(date)
        self.month = self.date_obj.month
        self.day = get_day(self.date_obj)
        self.year = self.date_obj.year

        self.url = self._build_url()
        self.soup = get_soup(self.url)
        self.results = []

    def get_url(self):
        """Returns the built url."""
        return self.url

    def get_soup(self):
        """Returns the HTML object"""
        return self.soup

    def fetch_result(self):
        """Return a list of urls for each search result."""
        results = self.soup.find_all('div', {'class': 'container container-small'})
        href = None
        is_match = False

        i = 0
        while i < len(results) and not is_match:
            result = results[i]
            anchor = result.find('a', {'rel': 'bookmark'})
            is_match = self._filter_results(result, anchor)
            href = anchor['href']

            i += 1

        try:
            page = get_soup(href)
        except (Exception):
            page = None

        # Return page if search is successful
        if href and page:
            return page
        else:
            raise PageNotFoundError(PAGE_ERROR)

    def _filter_results(self, result, anchor):
        """Filter search results by checking category titles and dates"""
        valid = True

        try:
            cat_tag = result.find('a', {'rel': 'category tag'}).string
            title = anchor.string.lower()
            date_tag = result.find('time').string
        except (AttributeError, TypeError):
            return False      
             
        if cat_tag != "Daily Ratings":
            valid = False
        if not date_in_range(self.date, date_tag, 5):
            valid = False
        if self.category == 'cable' and 'cable' not in title:
            valid = False
        elif self.category != 'cable' and 'cable' in title:
            valid = False

        return valid

    def _build_url(self):
        """Build url based on searching by date or by show."""
        url_params = [
            BASE_URL, self.category + ' ratings', self.day, self.year, self.month
        ]

        return SEARCH_URL.format(*url_params)

    def _assert_category(self, category):
        """Validate category argument"""
        category = category.lower()
        valid_categories = ['cable', 'broadcast', 'final', 'tv']
        assert_msg = "%s is not a valid category." % (category)
        assert (category in valid_categories), assert_msg



from .utils import convert_float, convert_time, convert_date
from .constants import FLOAT_ATTRIBUTES, NONFLOAT_ATTRIBUTES

class InvalidSortError(Exception):
    """Raise exception when sort parameter is not valid."""

    def __init__(self, sort):
        self.sort = sort

    def __str__(self):
        e = ('\'{0}\' is not a valid sort parameter.\n'
             'Valid parameters: {1}')
        return e.format(self.sort, str(FLOAT_ATTRIBUTES + NONFLOAT_ATTRIBUTES))


class Sorter(object):
    """Sort ratings objects by a specific attribute."""

    def __init__(self, rating, category, sort):
        """Sorting a ratings or show object will change the order of its entries.

        :param rating: A ratings object (cable or broadcast) or show object.
        :param category: cable or broadcast.
        :param sort: show, net, time, viewers, rating, or share (broadcast only)
        """
        if sort == 'share':
            assert category != 'cable', '"share" parameter for broadcast ratings only.'

        self.data = rating
        self.sort = sort

    def get_reverse(self):
        """By default, Cable entries are sorted by rating and Broadcast ratings are
        sorted by time.
        By default, float attributes are sorted from highest to lowest and non-float
        attributes are sorted alphabetically (show, net) or chronologically (time).
        """
        if self.sort in FLOAT_ATTRIBUTES:
            return True
        elif self.sort in NONFLOAT_ATTRIBUTES:
            return False
        else:
            raise InvalidSortError(self.sort)

    def sort_func(self, entry):
        """Return the key attribute to determine how data is sorted.
        Time will need to be converted to 24 hour time.
        In instances when float attributes will have an 'n/a' string, return 0.
        """
        key = entry[self.sort]

        if self.sort in FLOAT_ATTRIBUTES and not isinstance(key, float):
            return 0  # If value is 'n/a' string
        elif self.sort == 'time':
            return convert_time(key)
        elif self.sort == 'date':
            return convert_date(key)

        return key

    def sort_entries(self):
        """Get whether reverse is True or False. Return the sorted data."""
        return sorted(self.data, key=self.sort_func, reverse=self.get_reverse())



from datetime import datetime, timedelta

DATE_FMT = '%B %-d %Y'
YESTERDAY = (datetime.today() - timedelta(days=1)).strftime(DATE_FMT)

HEADERS = {'User-Agent': 'zap2it.py (https://github.com/sharibarboza/zap2it)'}

BASE_URL = 'http://tvbythenumbers.zap2it.com'
URL_FORMAT = '{0}/daily-ratings/{1}-{2}-{3}/'
SEARCH_URL = '{0}/?s={1}+{2}&year={3}&monthnum={4}&day&category=daily-ratings'

MONTHS = [
    'january', 'february', 'march', 'april',
    'may', 'june', 'july', 'august',
    'september', 'october', 'november', 'december'
]
SHORT_MONTHS = [
    'jan', 'feb', 'march', 'april',
    'may', 'june', 'july', 'aug',
    'sept', 'oct', 'nov', 'dec'
]

FLOAT_ATTRIBUTES = ['viewers', 'rating', 'share']
NONFLOAT_ATTRIBUTES = ['show', 'net', 'time', 'date']

PAGE_ERROR = "The page cannot be found."



#!/usr/bin/env python

from setuptools import setup

def readme():
    with open('README.rst') as f:
        return f.read()

setup(
    name='py_zap',
    version='1.2.1',
    description='Python scraper for accessing ratings from tvbythenumbers.zap2it.com',
    long_description=readme(),
    author='sharibarboza',
    author_email='barbozashari@gmail.com',
    url='https://github.com/sharibarboza/py_zap',
    download_url='https://github.com/sharibarboza/py_zap/archive/1.0.0.tar.gz',
    keywords=['zap2it', 'ratings', 'tv'],
    license='MIT License',
    packages=['py_zap'],
    install_requires=[
        'beautifulsoup4',
        'requests>=2.9.1'
    ]
)


#!/usr/bin/env python
from __future__ import print_function, absolute_import

import json
import requests
import calendar
import sys
from datetime import datetime, timedelta

from bs4 import BeautifulSoup

from .constants import HEADERS, MONTHS, SHORT_MONTHS, DATE_FMT

if sys.version_info[0] == 3:
    PY3 = True
else:
    PY3 = False

if PY3:
    sys.tracebacklimit = None
else:
    sys.tracebacklimit = 0


class PageNotFoundError(Exception):
    """Raise exception when page is not found."""
    pass


def to_json(data):
    """Return data as a JSON string."""
    return json.dumps(data, default=lambda x: x.__dict__, sort_keys=True, indent=4)

def convert_string(string, chars=None):
    """Remove certain characters from a string."""
    if chars is None:
        chars = [',', '.', '-', '/', ':', '  ']

    for ch in chars:
        if ch in string:
            string = string.replace(ch, ' ')
    return string

def convert_float(string):
    """Convert string into a float, otherwise return string."""
    try:
        return float(string)
    except (ValueError, TypeError):
        return string

def convert_time(time):
    """Convert a time string into 24-hour time."""
    split_time = time.split()
    try:
        # Get rid of period in a.m./p.m.
        am_pm = split_time[1].replace('.', '')
        time_str = '{0} {1}'.format(split_time[0], am_pm)
    except IndexError:
        return time
    try:
        time_obj = datetime.strptime(time_str, '%I:%M %p')
    except ValueError:
        time_obj = datetime.strptime(time_str, '%I %p')

    return time_obj.strftime('%H:%M %p')

#----------------------------------------------------------
# Date helpers
#----------------------------------------------------------

def convert_month(date, shorten=True, cable=True):
    """Replace month by shortening or lengthening it.

    :param shorten: Set to True to shorten month name.
    :param cable: Set to True if category is Cable.
    """
    month = date.split()[0].lower()
    if 'sept' in month:
        shorten = False if cable else True

    try:
        if shorten:
            month = SHORT_MONTHS[MONTHS.index(month)]
        else:
            month = MONTHS[SHORT_MONTHS.index(month)]
    except ValueError:
        month = month.title()

    return '{0} {1}'.format(month, ' '.join(date.split()[1:]))

def convert_date(date):
    """Convert string to datetime object."""
    date = convert_month(date, shorten=False)
    clean_string = convert_string(date)
    return datetime.strptime(clean_string, DATE_FMT.replace('-',''))

def get_day(date_obj):
    """Get the name of the day based on the date object."""
    return calendar.day_name[date_obj.weekday()]

def date_in_range(date1, date2, range):
    """Check if two date objects are within a specific range"""
    date_obj1 = convert_date(date1)
    date_obj2 = convert_date(date2)
    return (date_obj2 - date_obj1).days <= range

def inc_date(date_obj, num, date_fmt):
    """Increment the date by a certain number and return date object.
    as the specific string format.
    """
    return (date_obj + timedelta(days=num)).strftime(date_fmt)

def next_week(date_obj):
    """Return 7 days after the date object."""
    return inc_date(date_obj, 7, DATE_FMT)


def last_week(date_obj):
    """Return 7 days before the date object."""
    return inc_date(date_obj, -7, DATE_FMT)


#----------------------------------------------------------
# Parsing helpers
#----------------------------------------------------------

def get_soup(url):
    """Request the page and return the soup."""
    html = requests.get(url, stream=True, headers=HEADERS)
    if html.status_code != 404:
        return BeautifulSoup(html.content, 'html.parser')
    else:
        return None

def match_list(query_list, string):
    """Return True if all words in a word list are in the string.

    :param query_list: list of words to match
    :param string: the word or words to be matched against
    """
    # Get rid of 'the' word to ease string matching
    match = False
    index = 0
    string = ' '.join(filter_stopwords(string))

    if not isinstance(query_list, list):
        query_list = [query_list]

    while index < len(query_list):
        query = query_list[index]
        words_query = filter_stopwords(query)
        match = all(word in string for word in words_query)
        if match:
            break

        index += 1

    return match    

def filter_stopwords(phrase):
    """Filter out stop words and return as a list of words"""
    if not isinstance(phrase, list):
        phrase = phrase.split()

    stopwords = ['the', 'a', 'in', 'to']
    return [word.lower() for word in phrase if word.lower() not in stopwords]


def unescape_html(string):
    """Replace non-breaking space with white space."""
    return safe_unicode(string).replace('\xa0', ' ')


def safe_unicode(string):
    """If Python 2, replace non-ascii characters and return encoded string."""
    if not PY3:
        uni = string.replace(u'\u2019', "'")
        return uni.encode('utf-8')
        
    return string

def get_strings(soup, tag):
    """Get all the string children from an html tag."""
    tags = soup.find_all(tag)
    strings = [s.string for s in tags if s.string]
    return strings

def exceeded_limit(limit, length):
    """Check if the length exceeds a limit"""
    return True if limit and length >= limit else False
